@@ -8,7 +8,7 @@ this module.
 #==============================================================================
 __version__ = "$Revision: 1.66 $"
 __author__  = "K. Hilbert <Karsten.Hilbert@gmx.net>"
-__licence__ = "GPL (details at http://www.gnu.org)"
+__licence__ = "GPL v2 or later (details at http://www.gnu.org)"
 
 # standard libary
 import re, string, sys, time, datetime as pyDT, logging
@@ -31,6 +31,134 @@ from Gnumed.wxpython import gmGuiHelpers
 
 _log = logging.getLogger('gm.ui')
 
+#============================================================
+#class cIntervalMatchProvider(gmMatchProvider.cMatchProvider):
+#	"""Turns strings into candidate intervals."""
+#	def __init__(self):
+#
+#		gmMatchProvider.cMatchProvider.__init__(self)
+#
+#		self.setThresholds(aPhrase = 1, aWord = 998, aSubstring = 999)
+#		self.word_separators = None
+##		self.ignored_chars("""[?!."'\\(){}\[\]<>~#*$%^_]+""")
+#	#--------------------------------------------------------
+#	# external API
+#	#--------------------------------------------------------
+#	#--------------------------------------------------------
+#	# base class API
+#	#--------------------------------------------------------
+#	def getMatchesByPhrase(self, aFragment):
+#		intv = gmDateTime.str2interval(str_interval = aFragment)
+#
+#		if intv is None:
+#			return (False, [])
+#
+#		items = [{
+#			'data': intv,
+#			'field_label': gmDateTime.format_interval(intv, gmDateTime.acc_minutes),
+#			'list_label': gmDateTime.format_interval(intv, gmDateTime.acc_minutes)
+#		}]
+#
+#		return (True, items)
+#	#--------------------------------------------------------
+#	def getMatchesByWord(self, aFragment):
+#		return self.getMatchesByPhrase(aFragment)
+#	#--------------------------------------------------------
+#	def getMatchesBySubstr(self, aFragment):
+#		return self.getMatchesByPhrase(aFragment)
+#	#--------------------------------------------------------
+#	def getAllMatches(self):
+#		matches = (False, [])
+#		return matches
+#============================================================
+class cIntervalPhraseWheel(gmPhraseWheel.cPhraseWheel):
+
+	def __init__(self, *args, **kwargs):
+
+		gmPhraseWheel.cPhraseWheel.__init__(self, *args, **kwargs)
+		self.phrase_separators = None
+		self.display_accuracy = None
+	#--------------------------------------------------------
+	# phrasewheel internal API
+	#--------------------------------------------------------
+	def _update_candidates_in_picklist(self, val):
+		intv = gmDateTime.str2interval(str_interval = val)
+		if intv is None:
+			self._current_match_candidates = []
+		else:
+			self._current_match_candidates = [{
+				'data': intv,
+				'field_label': gmDateTime.format_interval(intv, gmDateTime.acc_minutes),
+				'list_label': gmDateTime.format_interval(intv, gmDateTime.acc_minutes)
+			}]
+		self._picklist.SetItems(self._current_match_candidates)
+	#---------------------------------------------------------
+#	def _on_lose_focus(self, event):
+#		# are we valid ?
+#		if len(self._data) == 0:
+#			self._set_data_to_first_match()
+#
+#		# let the base class do its thing
+#		super(cIntervalPhraseWheel, self)._on_lose_focus(event)
+	#--------------------------------------------------------
+	def _picklist_item2display_string(self, item=None):
+		intv = item['data']
+		if intv is not None:
+			return gmDateTime.format_interval (
+				interval = intv,
+				accuracy_wanted = self.display_accuracy
+			)
+		return item['field_label']
+	#--------------------------------------------------------
+	def _get_data_tooltip(self):
+		intv = self.GetData()
+		if intv is None:
+			return u''
+		return gmDateTime.format_interval (
+			interval = intv,
+			accuracy_wanted = self.display_accuracy
+		)
+	#--------------------------------------------------------
+	# external API
+	#--------------------------------------------------------
+	def SetValue(self, value):
+
+		if isinstance(value, pyDT.timedelta):
+			self.SetText(data = value, suppress_smarts = True)
+			return
+
+		if value is None:
+			value = u''
+
+		super(cIntervalPhraseWheel, self).SetValue(value)
+	#--------------------------------------------------------
+	def SetText(self, value=u'', data=None, suppress_smarts=False):
+
+		if data is not None:
+			if value.strip() == u'':
+				value = gmDateTime.format_interval (
+					interval = data,
+					accuracy_wanted = self.display_accuracy
+				)
+
+		super(cIntervalPhraseWheel, self).SetText(value = value, data = data, suppress_smarts = suppress_smarts)
+	#--------------------------------------------------------
+	def SetData(self, data=None):
+		if data is None:
+			super(cIntervalPhraseWheel, self).SetText(u'', None)
+			return
+
+		value = gmDateTime.format_interval (
+			interval = data,
+			accuracy_wanted = self.display_accuracy
+		)
+		super(cIntervalPhraseWheel, self).SetText(value = value, data = data)
+	#--------------------------------------------------------
+	def GetData(self):
+		if len(self._data) == 0:
+			self._set_data_to_first_match()
+
+		return super(cIntervalPhraseWheel, self).GetData()
 #============================================================
 class cCalendarDatePickerDlg(wx.Dialog):
 	"""Shows a calendar control from which the user can pick a date."""
@@ -109,18 +237,38 @@ class cDateMatchProvider(gmMatchProvider.cMatchProvider):
 	def getMatchesByPhrase(self, aFragment):
 		"""Return matches for aFragment at start of phrases."""
 		matches = gmDateTime.str2pydt_matches(str2parse = aFragment.strip())
-		if len(matches) > 0:
-			for match in matches:
-				if match['data'] is not None:
-					match['data'] = match['data'].replace (
-						hour = 11,
-						minute = 11,
-						second = 11,
-						microsecond = 111111
-					)
-			return (True, matches)
-		else:
+
+		if len(matches) == 0:
 			return (False, [])
+
+		items = []
+		for match in matches:
+			if match['data'] is None:
+				items.append ({
+					'data': None,
+					'field_label': match['label'],
+					'list_label': match['label']
+				})
+				continue
+
+			data = match['data'].replace (
+				hour = 11,
+				minute = 11,
+				second = 11,
+				microsecond = 111111
+			)
+			list_label = gmDateTime.pydt_strftime (
+				data,
+				format = '%A, %d. %B %Y (%x)',
+				accuracy = gmDateTime.acc_days
+			)
+			items.append ({
+				'data': data,
+				'field_label': match['label'],
+				'list_label': list_label
+			})
+
+		return (True, items)
 	#--------------------------------------------------------
 	def getMatchesByWord(self, aFragment):
 		"""Return matches for aFragment at start of words inside phrases."""
@@ -136,25 +284,26 @@ class cDateMatchProvider(gmMatchProvider.cMatchProvider):
 		matches = (False, [])
 		return matches
 
-		dlg = cCalendarDatePickerDlg(None)
-		# FIXME: show below parent
-		dlg.CentreOnScreen()
-
-		if dlg.ShowModal() == wx.ID_OK:
-			date = dlg.cal.Date
-			if date is not None:
-				if date.IsValid():
-					date = gmDateTime.wxDate2py_dt(wxDate = date).replace (
-						hour = 11,
-						minute = 11,
-						second = 11,
-						microsecond = 111111
-					)
-					lbl = gmDateTime.pydt_strftime(date, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days)
-					matches = (True, [{'data': date, 'label': lbl}])
-		dlg.Destroy()
-
-		return matches
+#		# consider this:
+#		dlg = cCalendarDatePickerDlg(None)
+#		# FIXME: show below parent
+#		dlg.CentreOnScreen()
+#
+#		if dlg.ShowModal() == wx.ID_OK:
+#			date = dlg.cal.Date
+#			if date is not None:
+#				if date.IsValid():
+#					date = gmDateTime.wxDate2py_dt(wxDate = date).replace (
+#						hour = 11,
+#						minute = 11,
+#						second = 11,
+#						microsecond = 111111
+#					)
+#					lbl = gmDateTime.pydt_strftime(date, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days)
+#					matches = (True, [{'data': date, 'label': lbl}])
+#		dlg.Destroy()
+#
+#		return matches
 #============================================================
 class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 
@@ -165,21 +314,18 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 		self.matcher = cDateMatchProvider()
 		self.phrase_separators = None
 
-		self.static_tooltip_extra = _('<ALT-C>: pick from calendar')
+		self.static_tooltip_extra = _('<ALT-C/K>: pick from (c/k)alendar')
 	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
-	def __text2timestamp(self, val=None):
-
-		if val is None:
-			val = self.GetValue().strip()
-
-		success, matches = self.matcher.getMatchesByPhrase(val)
-
-		if len(matches) == 1:
-			return matches[0]['data']
-
-		return None
+#	def __text2timestamp(self):
+#
+#		self._update_candidates_in_picklist(val = self.GetValue().strip())
+#
+#		if len(self._current_match_candidates) == 1:
+#			return self._current_match_candidates[0]['data']
+#
+#		return None
 	#--------------------------------------------------------
 	def __pick_from_calendar(self):
 		dlg = cCalendarDatePickerDlg(self)
@@ -210,19 +356,21 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	# phrasewheel internal API
 	#--------------------------------------------------------
 	def _on_lose_focus(self, event):
-		# are we valid ?
-		if self.data is None:
-			# no, so try
-			self.data = self.__text2timestamp()
+		# no valid date yet ?
+		if len(self._data) == 0:
+			self._set_data_to_first_match()
+			date = self.GetData()
+			if date is not None:
+				self.SetValue(gmDateTime.pydt_strftime(date, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days))
 
 		# let the base class do its thing
-		super(self.__class__, self)._on_lose_focus(event)
+		super(cDateInputPhraseWheel, self)._on_lose_focus(event)
 	#--------------------------------------------------------
-	def _picklist_selection2display_string(self):
-		data = self._picklist.GetSelectedItemData()
+	def _picklist_item2display_string(self, item=None):
+		data = item['data']
 		if data is not None:
 			return gmDateTime.pydt_strftime(data, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days)
-		return self._picklist.get_selected_item_label()
+		return item['field_label']
 	#--------------------------------------------------------
 	def _on_key_down(self, event):
 
@@ -233,14 +381,20 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 				self.__pick_from_calendar()
 				return
 
-		super(self.__class__, self)._on_key_down(event)
+		super(cDateInputPhraseWheel, self)._on_key_down(event)
 	#--------------------------------------------------------
 	def _get_data_tooltip(self):
-		if self.data is None:
+		if len(self._data) == 0:
+			return u''
+
+		date = self.GetData()
+		# if match provider only provided completions
+		# but not a full date with it
+		if date is None:
 			return u''
 
 		return gmDateTime.pydt_strftime (
-			self.data,
+			date,
 			format = '%A, %d. %B %Y (%x)',
 			accuracy = gmDateTime.acc_days
 		)
@@ -294,13 +448,13 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 			super(self.__class__, self).SetText(value = val, data = data)
 	#--------------------------------------------------------
 	def GetData(self):
-		if self.data is None:
-			self.data = self.__text2timestamp()
+		if len(self._data) == 0:
+			self._set_data_to_first_match()
 
 		return super(self.__class__, self).GetData()
 	#--------------------------------------------------------
 	def is_valid_timestamp(self, allow_empty=True):
-		if self.data is not None:
+		if len(self._data) > 0:
 			self.display_as_valid(True)
 			return True
 
@@ -317,26 +471,31 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 			self.display_as_valid(False)
 			return False
 
-		self.data = self.__text2timestamp()
-		if self.data is None:
+		# try to auto-snap to first match
+		self._set_data_to_first_match()
+		if len(self._data) == 0:
 			self.display_as_valid(False)
 			return False
 
+		date = self.GetData()
+		self.SetValue(gmDateTime.pydt_strftime(date, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days))
 		self.display_as_valid(True)
 		return True
 	#--------------------------------------------------------
 	# properties
 	#--------------------------------------------------------
 	def _get_date(self):
-		return self.data
+		return self.GetData()
 
 	def _set_date(self, date):
-		self.data = date.replace (
-			hour = 11,
-			minute = 11,
-			second = 11,
-			microsecond = 111111
-		)
+		raise AttributeError('._set_date not implemented')
+#		val = gmDateTime.pydt_strftime(date, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days)
+#		self.data = date.replace (
+#			hour = 11,
+#			minute = 11,
+#			second = 11,
+#			microsecond = 111111
+#		)
 
 	date = property(_get_date, _set_date)
 #============================================================
@@ -365,12 +524,20 @@ class cMatchProvider_FuzzyTimestamp(gmMatchProvider.cMatchProvider):
 	#--------------------------------------------------------
 	def getMatchesByPhrase(self, aFragment):
 		"""Return matches for aFragment at start of phrases."""
-		self.__now = mxDT.now()
 		matches = gmDateTime.str2fuzzy_timestamp_matches(aFragment.strip())
-		if len(matches) > 0:
-			return (True, matches)
-		else:
+
+		if len(matches) == 0:
 			return (False, [])
+
+		items = []
+		for match in matches:
+			items.append ({
+				'data': match['data'],
+				'field_label': match['label'],
+				'list_label': match['label']
+			})
+
+		return (True, items)
 	#--------------------------------------------------------
 	def getMatchesByWord(self, aFragment):
 		"""Return matches for aFragment at start of words inside phrases."""
@@ -382,7 +549,6 @@ class cMatchProvider_FuzzyTimestamp(gmMatchProvider.cMatchProvider):
 	#--------------------------------------------------------
 	def getAllMatches(self):
 		"""Return all items."""
-		# FIXME: popup calendar to pick from
 		return (False, [])
 #==================================================
 class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
@@ -395,6 +561,7 @@ class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
 		self.phrase_separators = None
 		self.selection_only = True
 		self.selection_only_error_msg = _('Cannot interpret input as timestamp.')
+		self.display_accuracy = None
 	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
@@ -415,16 +582,19 @@ class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
 		# are we valid ?
 		if self.data is None:
 			# no, so try
-			self.data = self.__text2timestamp()
+			date = self.__text2timestamp()
+			if date is not None:
+				self.SetValue(value = date.format_accurately(accuracy = self.display_accuracy))
+				self.data = date
 
 		# let the base class do its thing
 		gmPhraseWheel.cPhraseWheel._on_lose_focus(self, event)
 	#--------------------------------------------------------
-	def _picklist_selection2display_string(self):
-		data = self._picklist.GetSelectedItemData()
+	def _picklist_item2display_string(self, item=None):
+		data = item['data']
 		if data is not None:
-			return data.format_accurately()
-		return self._picklist.get_selected_item_label()
+			return data.format_accurately(accuracy = self.display_accuracy)
+		return item['field_label']
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
@@ -434,7 +604,7 @@ class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
 			if isinstance(data, pyDT.datetime):
 				data = gmDateTime.cFuzzyTimestamp(timestamp=data)
 			if value.strip() == u'':
-				value = data.format_accurately()
+				value = data.format_accurately(accuracy = self.display_accuracy)
 
 		gmPhraseWheel.cPhraseWheel.SetText(self, value = value, data = data, suppress_smarts = suppress_smarts)
 	#--------------------------------------------------------
@@ -444,7 +614,7 @@ class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
 		else:
 			if isinstance(data, pyDT.datetime):
 				data = gmDateTime.cFuzzyTimestamp(timestamp=data)
-			gmPhraseWheel.cPhraseWheel.SetText(self, value = data.format_accurately(), data = data)
+			gmPhraseWheel.cPhraseWheel.SetText(self, value = data.format_accurately(accuracy = self.display_accuracy), data = data)
 	#--------------------------------------------------------
 	def is_valid_timestamp(self):
 		if self.data is not None:
@@ -454,97 +624,17 @@ class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
 		if self.GetValue().strip() == u'':
 			return True
 
-		self.data = self.__text2timestamp()
-		if self.data is None:
+		date = self.__text2timestamp()
+		if date is None:
 			return False
 
-		return True
-#==================================================
-class cTimeInput(wx.TextCtrl):
-	def __init__(self, parent, *args, **kwargs):
-		if len(args) < 2:
-			if not kwargs.has_key('value'):
-				kwargs['value'] = _('enter time here')
-		wx.TextCtrl.__init__(
-			self,
-			parent,
-			*args,
-			**kwargs
+		self.SetText (
+			value = date.format_accurately(accuracy = self.display_accuracy),
+			data = date,
+			suppress_smarts = True
 		)
-#==================================================
-#class cDateInputCtrl(wx.DatePickerCtrl):
-#
-#	#----------------------------------------------
-#	def SetValue(self, value):
-#		"""Set either datetime.datetime or wx.DateTime"""
-#
-#		if isinstance(value, (pyDT.date, pyDT.datetime)):
-#			value = gmDateTime.py_dt2wxDate(py_dt = value, wx = wx)
-#
-#		elif value is None:
-#			value = wx.DefaultDateTime
-#
-#		wx.DatePickerCtrl.SetValue(self, value)
-#	#----------------------------------------------
-#	def GetValue(self, as_pydt=False, invalid_as_none=False):
-#		"""Returns datetime.datetime values"""
-#
-#		# datepicker can fail to pick up user changes by keyboard until
-#		# it has lost focus, so do that but also set the focus back to us,
-#		# now, this is a side-effect (after .GetValue focus will be
-#		# here) but at least it is predictable ...
-#		self.Navigate()
-#		self.SetFocus()
-#		value = wx.DatePickerCtrl.GetValue(self)
-#
-#		if value is None:
-#			return None
-#
-#		# manage null dates (useful when wx.DP_ALLOWNONE is set)
-#		if not value.IsValid():
-#			if invalid_as_none:
-#				return None
-#			else:
-#				return value
-#
-#		self.SetBackgroundColour(gmPhraseWheel.color_prw_valid)
-#		self.Refresh()
-#
-#		if not as_pydt:
-#			return value
-#
-#		return gmDateTime.wxDate2py_dt(value)
-#	#----------------------------------------------
-#	# def convenience wrapper
-#	#----------------------------------------------
-#	def is_valid_timestamp(self, allow_none=True, invalid_as_none=False):
-#		val = self.GetValue(as_pydt = False, invalid_as_none = invalid_as_none)
-#
-#		if val is None:
-#			if allow_none:
-#				valid = True
-#			else:
-#				valid = False
-#		else:
-#			valid = val.IsValid()
-#
-#		if valid:
-#			self.SetBackgroundColour(gmPhraseWheel.color_prw_valid)
-#		else:
-#			self.SetBackgroundColour(gmPhraseWheel.color_prw_invalid)
-#
-#		self.Refresh()
-#		return valid
-#	#----------------------------------------------
-#	def get_pydt(self):
-#		return self.GetValue(as_pydt = True)
-#	#----------------------------------------------
-#	def display_as_valid(self, valid=True):
-#		if valid is True:
-#			self.SetBackgroundColour(gmPhraseWheel.color_prw_valid)
-#		else:
-#			self.SetBackgroundColour(gmPhraseWheel.color_prw_invalid)
-#		self.Refresh()
+
+		return True
 #==================================================
 # main
 #--------------------------------------------------
@@ -553,7 +643,7 @@ if __name__ == '__main__':
 	if len(sys.argv) < 2:
 		sys.exit()
 
-	if sys.argv[2] != 'test':
+	if sys.argv[1] != 'test':
 		sys.exit()
 
 	gmI18N.activate_locale()
@@ -568,25 +658,26 @@ if __name__ == '__main__':
 		val = None
 		while val != 'exit':
 			print "************************************"
-			val = raw_input('Enter date fragment: ')
+			val = raw_input('Enter date fragment ("exit" to quit): ')
 			found, matches = mp.getMatches(aFragment=val)
 			for match in matches:
+				#print match
 				print match['label']
 				print match['data']
 				print "---------------"
 	#--------------------------------------------------------
-	def test_gui():
-		app = wx.PyWidgetTester(size = (200, 300))
+	def test_fuzzy_picker():
+		app = wx.PyWidgetTester(size = (300, 40))
 		app.SetWidget(cFuzzyTimestampInput, id=-1, size=(180,20), pos=(10,20))
 		app.MainLoop()
-#	#--------------------------------------------------------
-#	def test_picker():
-#		app = wx.PyWidgetTester(size = (200, 300))
-#		app.SetWidget(cDateInputCtrl, id=-1, size=(180,20), pos=(10,20))
-#		app.MainLoop()
+	#--------------------------------------------------------
+	def test_picker():
+		app = wx.PyWidgetTester(size = (300, 40))
+		app.SetWidget(cDateInputPhraseWheel, id=-1, size=(180,20), pos=(10,20))
+		app.MainLoop()
 	#--------------------------------------------------------
 	#test_cli()
-	#test_gui()
+	#test_fuzzy_picker()
 	test_picker()
 
 #==================================================
